import gtk, gobject
import icon, pixmaps

tooltips = gtk.Tooltips()

LEFT = 1
RIGHT = 2

class MenuBox(gtk.EventBox):
	
	def __init__(self, has_arrow = True, edge = 0, pos_func = None):
		assert (edge == 0 or edge == icon.EDGE_LEFT or edge == icon.EDGE_RIGHT 
				or edge == icon.EDGE_TOP or edge == icon.EDGE_BOTTOM)
		gtk.EventBox.__init__(self)
		self.menu = None
		
		# function to position the menu
		self.pos_func = pos_func
		
		# panel side
		self.edge = edge

		self.image = gtk.Image()
		self.set_has_arrow(has_arrow)
		self.add(self.image)

		self.connect("button-press-event", self.__button_press_event)

	def __button_press_event(self, widget, event):
		menu = self.get_menu()
		if not menu:
			return
		menu.popup(None, None, self.pos_func, event.button, event.time)

	def get_menu(self):
		if not self.menu:
			self.menu = self.create_menu()
		return self.menu

	def create_menu(self):
		"""Override this to set up the menu."""
		return None

	def forget_menu(self):
		self.menu = None
		
	def set_has_arrow(self, has_arrow):
		self.has_arrow = has_arrow
		if has_arrow:
			if self.edge == icon.EDGE_LEFT:
				pixmap = pixmaps.right
			elif self.edge == icon.EDGE_RIGHT:
				pixmap = pixmaps.left
			elif self.edge == icon.EDGE_TOP:
				pixmap = pixmaps.down
			else:
				pixmap = pixmaps.up
		else:
			pixmap = pixmaps.none
		pixbuf = gtk.gdk.pixbuf_new_from_xpm_data(pixmap)
		self.image.set_from_pixbuf(pixbuf)


class Box:
	
	def __init__(self, name, menubox_class = MenuBox, has_arrow = True, 
				edge = 0, pos_func = None):
		assert issubclass(menubox_class, MenuBox)
		assert (edge == 0 or edge == icon.EDGE_LEFT or edge == icon.EDGE_RIGHT 
				or edge == icon.EDGE_TOP or edge == icon.EDGE_BOTTOM)
		self.separators = LEFT|RIGHT
		self.menus = LEFT|RIGHT
		self.vertical = (edge == icon.EDGE_LEFT or edge == icon.EDGE_RIGHT)
		if self.vertical:
			self.separator_left = gtk.HSeparator()
			self.separator_right = gtk.HSeparator()
			self.set_size_request(8, -1)
		else:
			self.separator_left = gtk.VSeparator()
			self.separator_right = gtk.VSeparator()
			self.set_size_request(-1, 8)

		self.menubox_left = menubox_class(has_arrow, edge, pos_func)
		self.menubox_right = menubox_class(has_arrow, edge, pos_func)
		tooltips.set_tip(self.menubox_left, name)
		tooltips.set_tip(self.menubox_right, name)

		if self.vertical:
			self.box = gtk.VBox()
		else:
			self.box = gtk.HBox()

		self.pack_start(self.separator_left)
		self.pack_start(self.menubox_left)
		self.pack_start(self.box)
		self.pack_start(self.menubox_right)
		self.pack_start(self.separator_right)
		
	def set_has_arrow(self, has_arrow):
		self.menubox_left.set_has_arrow(has_arrow)
		self.menubox_right.set_has_arrow(has_arrow)

	def show_separators(self, separators):
		if separators & LEFT:
			self.separator_left.show()
		if separators & RIGHT:
			self.separator_right.show()
		self.separators |= separators
		
	def hide_separators(self, separators):
		if separators & LEFT:
			self.separator_left.hide()
		if separators & RIGHT:
			self.separator_right.hide()
		self.separators &= ~separators
		
	def set_menus(self, menus):
		self.show_menus(menus)
		self.hide_menus(~menus)

	def show_menus(self, menus):
		if menus & LEFT:
			self.menubox_left.show()
		if menus & RIGHT:
			self.menubox_right.show()
		self.menus |= menus 

	def hide_menus(self, menus):
		if menus & LEFT:
			self.menubox_left.hide()
		if menus & RIGHT:
			self.menubox_right.hide()
		self.menus &= ~menus

	def get_box(self):
		return self.box
		
	def set_icon_size(self, size):
		if self.vertical:
			self.set_size_request(int(size*1.5)+4, -1)
		else:
			self.set_size_request(-1, int(size*1.5)+4)
			
	def is_vertical(self):
		return self.vertical


class VBox(gtk.VBox, Box):
	
	def __init__(self, name, menubox_class = MenuBox, 
				has_arrow = True, edge = 0, pos_func = None):
		assert issubclass(menubox_class, MenuBox)
		assert edge == 0 or edge == icon.EDGE_LEFT or edge == icon.EDGE_RIGHT
		gtk.VBox.__init__(self)
		Box.__init__(self, name, menubox_class, has_arrow, edge, pos_func)
		

class HBox(gtk.HBox, Box):
	
	def __init__(self, name, menubox_class = MenuBox, 
				has_arrow = True, edge = 0, pos_func = None):
		assert issubclass(menubox_class, MenuBox)
		assert edge == 0 or edge == icon.EDGE_TOP or edge == icon.EDGE_BOTTOM
		gtk.HBox.__init__(self)
		Box.__init__(self, name, menubox_class, has_arrow, edge, pos_func)


def TrayBox(name, menubox_class = MenuBox, 
			has_arrow = True, edge = 0, pos_func = None):
	assert issubclass(menubox_class, MenuBox)
	assert (edge == 0 or edge == icon.EDGE_LEFT or edge == icon.EDGE_RIGHT 
			or edge == icon.EDGE_TOP or edge == icon.EDGE_BOTTOM)
	if edge == 0 or edge == icon.EDGE_TOP or edge == icon.EDGE_BOTTOM:
		return HBox(name, menubox_class, has_arrow, edge, pos_func)
	else:
		return VBox(name, menubox_class, has_arrow, edge, pos_func)

