import gtk, gobject
import pixmaps, pixbuf_helper

EDGE_LEFT = 1
EDGE_RIGHT = 2
EDGE_TOP = 3
EDGE_BOTTOM = 4

ACTION_HIDE = 1
ACTION_DESTROY = 2
ACTION_SHOW = 3


# For debugging, a global unique index that every Icon gets
idx=0

class Icon(gtk.EventBox):
	"""A widget containing an image that can be zoomed out and in. If the mouse
	pointer moves over the widget, it is zoomed in.""" 
	
	def __init__(self, size, pixbuf = None, edge = 0, 
				zoom_on_mouse_over = True, has_arrow = False):
		"""Creates a new Icon. edge must be one of EDGE_TOP, EDGE_BOTTOM, 
		EDGE_LEFT or EDGE_RIGHT. It defines the edge where the icon is not 
		shrunk when the mouse pointer comes near it."""
		global idx
		gtk.EventBox.__init__(self)
		self.add_events(gtk.gdk.POINTER_MOTION_MASK)
		self.image = gtk.Image()
		self.add(self.image)
		self.image.show()
		self.connect("enter-notify-event", self.__motion)
		self.connect("motion-notify-event", self.__motion)
		self.connect("leave-notify-event", self.__leave_notify_event)
		self.connect("button-release-event", self.__button_release_event)
		self.zoom_on_mouse_over = zoom_on_mouse_over 
		#self.move_others = move_others
		self.mouse_over = False
		self.mouse_over_ratio = 0.0
		
		if edge == EDGE_LEFT:
			pixmap = pixmaps.right
		elif edge == EDGE_RIGHT:
			pixmap = pixmaps.left
		elif edge == EDGE_TOP:
			pixmap = pixmaps.down
		else:
			pixmap = pixmaps.up

		self.arrow = gtk.gdk.pixbuf_new_from_xpm_data(pixmap)
		self.has_arrow = has_arrow
		self.had_arrow = False # True if the icon had an arrow before being hidden
		
		self.emblem = None
		self.emblem_scaled = None
		self.old_emblem = None
		
		self.fade_event = 0
		self.arrow_alpha = 0
		self.emblem_alpha = 0

		self.zoom_current_size = 1
		self.zoom_event = 0
		self.zoom_action = 0

		self.size = size
		self.max_size = int(size*1.5)
		self.canvas = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 
										self.max_size, self.max_size)

		self.edge = edge

		self.blink_event = 0
		self.blink_state = gtk.STATE_NORMAL

		self.pixbuf = None
		self.set_pixbuf(pixbuf)
		self.idx = idx
		idx += 1

	def set_emblem(self, emblem):
		if emblem and self.emblem != emblem:
			self.emblem_scaled = pixbuf_helper.scale_pixbuf_to_size(emblem,
																self.max_size/3)
		self.emblem = emblem
		if self.fade_event == 0:
			self.fade_event = gobject.timeout_add(7, self._fade)

	def set_has_arrow(self, has_arrow):
		#print "set_has_arrow(%d)" % has_arrow
		if self.has_arrow == has_arrow:
			return
		self.has_arrow = has_arrow
		if self.fade_event == 0:
			#print "self.fade_event = gobject.timeout_add(7, self._fade)"
			self.fade_event = gobject.timeout_add(7, self._fade)

	def _fade(self):
		#print "_fade()"
		#print "zoom action: %d" % self.zoom_action
		# if we are zooming in at the moment, defer the fading in
		if self.zoom_event > 0 and self.zoom_action == ACTION_SHOW:
			return True
		self._scale(self.zoom_current_size)
		arrow_fade_finished = False
		emblem_fade_finished = False
		if self.has_arrow and self.arrow_alpha == 255:
			# arrow has been faded in
			arrow_fade_finished = True
		if not self.has_arrow and self.arrow_alpha == 0:
			# arrow has been faded out
			arrow_fade_finished = True
		if self.emblem and self.emblem_alpha == 196:
			# emblem has been faded in
			emblem_fade_finished = True
		if not self.emblem and self.emblem_alpha == 0:
			# arrow has been faded out
			emblem_fade_finished = True
		if arrow_fade_finished and emblem_fade_finished:
			self.fade_event = 0
			return False # emblem and arrow have been faded out or in
		if self.has_arrow:
			self.arrow_alpha = min(self.arrow_alpha + 5, 255)
		else:
			self.arrow_alpha = max(self.arrow_alpha - 5, 0)
		if self.emblem:
			self.emblem_alpha = min(self.emblem_alpha + 5, 196)
		else:
			self.emblem_alpha = max(self.emblem_alpha - 5, 0)
		return True

	def set_zoom_on_mouse_over(self, zoom_on_mouse_over):
		self.zoom_on_mouse_over = zoom_on_mouse_over
		if not zoom_on_mouse_over:
			self.mouse_over_ratio = 0.0

	def set_blinking(self, blinking):
		if not blinking:
			if self.blink_event != 0:
				gobject.source_remove(self.blink_event)
				self.blink_event = 0
				self.set_state(gtk.STATE_NORMAL)
		else:
			if self.blink_event == 0:
				self.blink_event = gobject.timeout_add(250, self._blink)

	def _blink(self):
		if self.blink_state == gtk.STATE_SELECTED:
			self.blink_state = gtk.STATE_NORMAL
		elif self.blink_state == gtk.STATE_NORMAL:
			self.blink_state = gtk.STATE_SELECTED
		self.set_state(self.blink_state)
		return True

	def is_mouse_over(self):
		"""Returns True if the mouse pointer is over the Icon."""
		return self.mouse_over

	def set_pixbuf(self, pixbuf):
		"""Sets the pixbuf to use.""" 
		if pixbuf == None or pixbuf == self.pixbuf:
			return
		self.pixbuf = pixbuf_helper.scale_pixbuf_to_size(pixbuf, self.max_size, 
													scale_up = False)
		self._scale(self.zoom_current_size)

	def set_size(self, size, max_size = 0):
		"""Sets the Icon's size. This is the size when not zoomed in."""
		assert size > 0
		if size == self.size:
			return
		if not self.pixbuf:
			return
		self.size = size
		if self.zoom_on_mouse_over:
			size += size*self.mouse_over_ratio
		action = 0
		if max_size > 0 and max_size != self.max_size:
			self.max_size = max_size
			self.canvas = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 
										self.max_size, self.max_size)
			
			action = ACTION_SHOW
		self.set_emblem(self.emblem)
		self.zoom(size, action)

	def _scale(self, size):
		if not self.pixbuf:
			return
		#if size == self.max_size:
		#	if self.zoom_event > 0:
		#		gobject.source_remove(self.zoom_event)
		#		self.zoom_event = 0
		#		self.zoom_action = 0
		#	return
		size = min(size, self.max_size)
		pixbuf = pixbuf_helper.scale_pixbuf_to_size(self.pixbuf, size)
		if (self.emblem or self.emblem_alpha > 0 
			or self.has_arrow or self.arrow_alpha > 0):
			self.canvas.fill(0x000000)
			width = pixbuf.get_width()
			height = pixbuf.get_height()
			x = self.max_size/2-width/2
			y = self.max_size/2-height/2
			#scale_x = float(size)/float(width)
			#scale_y = float(size)/float(height)
			pixbuf.composite(self.canvas, x, y, width, height, x, y, 
									1.0, 1.0, gtk.gdk.INTERP_BILINEAR, 255)
			if self.emblem or self.emblem_alpha > 0:
				width = self.max_size/3
				height = width
				self.emblem_scaled.composite(self.canvas, 0, 0, 
									width, height, 
									0, 0, 
									1.0, 1.0, gtk.gdk.INTERP_BILINEAR, 
									self.emblem_alpha)
			if self.has_arrow or self.arrow_alpha > 0:
				width = self.arrow.get_width()
				height = self.arrow.get_height()
				x = 0
				y = 0
				if (self.edge == 0 
					or self.edge == EDGE_TOP or self.edge == EDGE_BOTTOM):
					x = self.max_size/2 - width/2
					if self.edge == EDGE_TOP:
						y = self.max_size - height
				if self.edge == EDGE_LEFT or self.edge == EDGE_RIGHT:
					y = self.max_size/2 - height/2
					if self.edge == EDGE_LEFT:
						x = self.max_size - width
	
				self.arrow.composite(self.canvas, x, y, 
									width, height, 
									x, y, 
									1.0, 1.0, gtk.gdk.INTERP_BILINEAR, 
									self.arrow_alpha)
			self.image.set_from_pixbuf(self.canvas)
		else:
			self.image.set_from_pixbuf(pixbuf)
		self.zoom_current_size = size

	def destroy(self):
		# Unset size_request so it zooms out smoothly, collapsing all other icons
		# around it.
		self.set_size_request(-1, -1)
		self.zoom(1, ACTION_DESTROY)

	def hide(self):
		# Unset size_request so it zooms out smoothly, collapsing all other icons
		# around it
		self.set_size_request(-1, -1)
		self.zoom(1, ACTION_HIDE)

	def show(self):
		if not int(self.get_property("visible")):
			# Unset size_request so it zooms in smoothly, pushing away all other
			# icons around it
			self.set_size_request(-1, -1)
			self._scale(1)
			gtk.EventBox.show(self)
		self.zoom(self.size, ACTION_SHOW)

	def zoom(self, target_size, action = 0):
		if self.zoom_action == ACTION_DESTROY:
			return # Don't stop icon from being destroyed.
		if (self.zoom_action == ACTION_HIDE 
			and action != ACTION_SHOW and action != ACTION_DESTROY):
			# Don't stop icon from being hidden,
			# unless it's to be shown or destroyed 
			return 

		self.zoom_target_size = int(target_size)

		if self.zoom_event > 0:
			# If show event is cancelled, set size request now.
			if self.zoom_action == ACTION_SHOW and action != ACTION_SHOW:
				self._zoom_action(ACTION_SHOW)
			self.zoom_action = action
			# We have set the new zoom_target_size and zoom_action, so the existing
			# zoom_event will take us there automatically, no need to start a new
			# zoom_event.
			return
		#elif self.zoom_target_size == self.zoom_current_size:
		#	self._zoom_action(action)
		#	return
		self.zoom_action = action
		self.zoom_event = gobject.timeout_add(5, self._zoom)

	def _zoom(self):
		#print "_zoom()"
		#print "zoom action: %d" % self.zoom_action
		# if the icon is being hidden or destroyed and the arrow is still 
		# visible, fade out the arrow
		if ((self.zoom_action == ACTION_DESTROY or self.zoom_action == ACTION_HIDE) 
			and self.arrow_alpha > 0):
			self.had_arrow = True
			self.set_has_arrow(False)
			return True
		# if the icon is being hidden or destroyed and the emblem is still 
		# visible, fade out the emblem
		if ((self.zoom_action == ACTION_DESTROY or self.zoom_action == ACTION_HIDE) 
			and self.emblem_alpha > 0):
			self.old_emblem = self.emblem
			self.set_emblem(None)
			return True
		target_size = self.zoom_target_size
		if target_size > self.zoom_current_size:
			self._scale(self.zoom_current_size + 1)
		elif target_size < self.zoom_current_size:
			self._scale(self.zoom_current_size - 1)

		#print "target size: %d" % target_size
		#print "current size: %d" % self.zoom_current_size
		if target_size == self.zoom_current_size:
			self._zoom_action(self.zoom_action)
			self.zoom_event = 0
			self.zoom_action = 0
			return False
		return True

	def _zoom_action(self, action):
		if action == ACTION_HIDE:
			gtk.EventBox.hide(self)
		elif action == ACTION_DESTROY:
			gtk.EventBox.destroy(self)
		elif action == ACTION_SHOW:
			if self.had_arrow:
				self.set_has_arrow(True)
				self.had_arrow = False
			if self.old_emblem:
				self.set_emblem(self.old_emblem)
				self.old_emblem = None
			#if not self.move_others:
			self.set_size_request(self.max_size, self.max_size)

	def __motion(self, widget, event):
		if self.zoom_action != 0:
			return
		i, i, w, h, i = self.window.get_geometry()
		if event.x < 0 or event.y < 0 or event.x >= w or event.y >= h:
			self.mouse_over = False
			return
		self.mouse_over = True
		if not self.zoom_on_mouse_over:
			return
		hw = float(w)/2.0
		hh = float(h)/2.0
		fract_x = (hw - (1.0/hw)*(event.x-hw)**2) / hw
		fract_y = (hh - (1.0/hh)*(event.y-hh)**2) / hh
		if (self.edge == EDGE_TOP and event.y < hh 
				or self.edge == EDGE_BOTTOM and event.y > hh):
			fract = fract_x
		elif (self.edge == EDGE_LEFT and event.x < hw 
				or self.edge == EDGE_RIGHT and event.x > hw):
			fract = fract_y
		else:
			fract = fract_x * fract_y
		self.mouse_over_ratio = fract/2.0
		size = int(self.size + self.size*(self.mouse_over_ratio))
		if size < self.size or size == self.zoom_current_size:
			return
		self.zoom(size)

	def __leave_notify_event(self, widget, event):
		if event.mode != gtk.gdk.CROSSING_NORMAL:
			return
		self._leave()

	def _leave(self):
		self.mouse_over = False
		self.mouse_over_ratio = 0.0
		if not self.zoom_on_mouse_over:
			return
		self.zoom(self.size)

	def __button_release_event(self, widget, event):
		px, py = self.get_pointer()
		w, h = self.get_size_request()
		if px < 0 or py < 0 or px > w or py > h:
			 self._leave()
		return False
